<?php

namespace Drupal\migrate_wizard\Plugin\migrate\process;

use Drupal\file\Entity\File;
use Drupal\migrate\MigrateSkipProcessException;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\Row;
use Drupal\Core\File\FileSystemInterface;

/**
 * Get file.
 *
 * @MigrateProcessPlugin(
 *   id = "get_file"
 * )
 */
class GetFile extends ProcessPluginBase {

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected $migrationUrlFilesProd;

  /**
   * {@inheritdoc}
   */
  protected $fileSystem;

  /**
   * {@inheritdoc}
   */
  protected $entityManager;

  /**
   * {@inheritdoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {

    $this->config = \Drupal::configFactory()->getEditable('migrate_wizard.settings');
    $this->migrationUrlFilesProd = $this->config->get('migrate_wizard_settings')['migration_url_files_prod'];
    $this->fileSystem = \Drupal::service('file_system');
    $this->entityManager = \Drupal::service('entity_type.manager');

    return $this->createFile($value, $row);
  }

  /**
   * Create file.
   */
  protected function createFile(array $file_data, $row) {

    $file_url = str_replace('public://', $this->migrationUrlFilesProd, $file_data['uri']);

    $directory = str_replace($this->migrationUrlFilesProd, '', $file_url);
    if (basename($file_url) != $directory) {
      $directory = str_replace(basename($file_url), '', $directory);
      $directory = 'public://' . $directory;
    }
    else {
      $directory = 'public://';
    }

    $uri = $directory . basename($file_url);

    $this->fileSystem->prepareDirectory($directory, FileSystemInterface:: CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);

    if (file_exists($directory . '/' . basename($file_url))) {

      $files = $this->entityManager->getStorage('file')->loadByProperties(['uri' => $uri]);

      $file = reset($files) ?: NULL;
      if ($file == NULL) {
        // Create file with File::create.
        $file = File::create([
          'uri' => $uri,
          'uid' => 1,
          'status' => 1,
        ]);
        $file->save();
      }
    }
    else {
      stream_context_set_default( [
          'ssl' => [
              'verify_peer' => false,
              'verify_peer_name' => false,
          ],
      ]);
      $url_downloadable = get_headers($file_url);

      // Check the response code.
      if (isset($url_downloadable[0]) && $url_downloadable[0] == 'HTTP/1.1 200 OK') {
        $file = $this->system_retrieve_file($file_url, $directory, TRUE, FileSystemInterface::EXISTS_REPLACE);
      }
      else {
        \Drupal::logger('migrate_wizard')->notice(' File not found: ' . $file_url.' in node: '.$row->getSourceProperty('nid'));
        throw new MigrateSkipProcessException($file_url . ' Error ' . $url_downloadable[0]);
      }

    }

    if ($file instanceof \Drupal\file\FileInterface) {
      if ($file_data["type"] == 'file') {
        $value_file = [
          'target_id' => $file->id(),
          'description' => $file_data['description'],
        ];
      }
      elseif ($file_data["type"] == 'image') {
        $value_file = [
          'target_id' => $file->id(),
          'alt' => $file_data['alt'],
          'title' => $file_data['title'],
          'width' => $file_data['width'],
          'height' => $file_data['height'],
        ];
      }
      $value_file['type'] = $file_data['type'];
      return $value_file;
    }else{
      \Drupal::logger('migrate_wizard')->notice(' File not found: ' . $file_url.' in node: '.$row->getSourceProperty('nid'));
    }
  }

  /**
 * Attempts to get a file using Guzzle HTTP client and to store it locally.
 *
 * @param string $url
 *   The URL of the file to grab.
 * @param string $destination
 *   Stream wrapper URI specifying where the file should be placed. If a
 *   directory path is provided, the file is saved into that directory under
 *   its original name. If the path contains a filename as well, that one will
 *   be used instead.
 *   If this value is omitted, the site's default files scheme will be used,
 *   usually "public://".
 * @param bool $managed
 *   If this is set to TRUE, the file API hooks will be invoked and the file is
 *   registered in the database.
 * @param int $replace
 *   Replace behavior when the destination file already exists:
 *   - FileSystemInterface::EXISTS_REPLACE: Replace the existing file.
 *   - FileSystemInterface::EXISTS_RENAME: Append _{incrementing number} until
 *     the filename is unique.
 *   - FileSystemInterface::EXISTS_ERROR: Do nothing and return FALSE.
 *
 * @return mixed
 *   One of these possibilities:
 *   - If it succeeds and $managed is FALSE, the location where the file was
 *     saved.
 *   - If it succeeds and $managed is TRUE, a \Drupal\file\FileInterface
 *     object which describes the file.
 *   - If it fails, FALSE.
 */
function system_retrieve_file($url, $destination = NULL, $managed = FALSE, $replace = FileSystemInterface::EXISTS_RENAME) {
  $parsed_url = parse_url($url);
  /** @var \Drupal\Core\File\FileSystemInterface $file_system */
  $file_system = \Drupal::service('file_system');
  if (!isset($destination)) {
    $path = $file_system->basename($parsed_url['path']);
    $path = \Drupal::config('system.file')->get('default_scheme') . '://' . $path;
    $path = \Drupal::service('stream_wrapper_manager')->normalizeUri($path);
  }
  else {
    if (is_dir($file_system->realpath($destination))) {
      // Prevent URIs with triple slashes when glueing parts together.
      $path = str_replace('///', '//', "$destination/") . \Drupal::service('file_system')->basename($parsed_url['path']);
    }
    else {
      $path = $destination;
    }
  }
  try {
    $data = (string) \Drupal::httpClient()
      ->get($url, ['verify' => FALSE])
      ->getBody();
    if ($managed) {
      /** @var \Drupal\file\FileRepositoryInterface $file_repository */
      $file_repository = \Drupal::service('file.repository');
      $local = $file_repository->writeData($data, $path, $replace);
    }
    else {
      $local = $file_system->saveData($data, $path, $replace);
    }
  }
  catch (TransferException $exception) {
    \Drupal::messenger()->addError(t('Failed to fetch file due to error "%error"', ['%error' => $exception->getMessage()]));
    return FALSE;
  }
  catch (FileException | InvalidStreamWrapperException $e) {
    \Drupal::messenger()->addError(t('Failed to save file due to error "%error"', ['%error' => $e->getMessage()]));
    return FALSE;
  }
  if (!$local) {
    \Drupal::messenger()->addError(t('@remote could not be saved to @path.', ['@remote' => $url, '@path' => $path]));
  }

  return $local;
}

}
