<?php

namespace Drupal\migrate_wizard\Plugin\migrate\source;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Database;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\State\StateInterface;
use Drupal\migrate\Plugin\migrate\source\SqlBase;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\Row;
use Drupal\migrate_wizard\FieldTypeMWBase;
use Drupal\migrate_wizard\FieldTypeMWManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Wildcard plugin to migrate content.
 *
 * @MigrateSource(
 *   id = "wildcard",
 * )
 */
class Wildcard extends FieldTypeMWBase {

  /**
   * The config of content type to import.
   *
   * @var string
   */
  protected $config;

  /**
   * The config of content type to import.
   *
   * @var string
   */
  protected $configFactory;

  /**
   * Array of bundles.
   *
   * @var array
   */
  public $bundle;

  /**
   * Array of nid or vid destiny type.
   *
   * @var string
   */
  protected $destinyType;

  /**
   * Array of machine name of origin to import.
   *
   * @var string
   */
  protected $originType;

  /**
   * Query Sql to add conditions.
   *
   * @var Query
   */
  public $query;

  /**
   * The left join table.
   *
   * @var string
   */
  protected $leftJoinTable;

  /**
   * The current config of content to import.
   *
   * @var array
   */
  protected $currentConfig;


  /**
   * {@inheritdoc}
   */
  protected $fieldTypeMWManager;

  /**
   * {@inheritdoc}
   */
  protected $leftTable;

  /**
   * {@inheritdoc}
   */
  protected $leftTableId;

  /**
   * {@inheritdoc}
   */
  protected $entityType;

  /**
   * {@inheritdoc}
   */
  protected $translation;
  /**
   * Constructor.
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              MigrationInterface $migration,
                              StateInterface $state,
                              ConfigFactoryInterface $config_factory,
                              FieldTypeMWManagerInterface $field_type_mw_manager
                             ) {
    $this->state = $state;
    $current_source_config = $migration->getSourceConfiguration();
    $this->config = $config_factory->getEditable('migrate_wizard.settings');
    $this->bundle = $current_source_config['bundle'];
    if(isset($current_source_config["constants"]["translations"]) && $current_source_config["constants"]["translations"]){
      $this->bundle = str_replace('_'.$current_source_config["constants"]["langcode"], '', $current_source_config['bundle']);
    }
    $this->currentConfig = $this->config->get($this->bundle);

    $this->translation = ['und', 'es'];
    if(isset($this->currentConfig[array_key_first($this->currentConfig)]["languages"])){
      $this->translation = ['und', $this->currentConfig[array_key_first($this->currentConfig)]["languages"]["default"]];
    }

    if(isset($configuration["constants"]["translations"]) && $configuration["constants"]["translations"]){
      $this->translation = [$configuration["constants"]["langcode"]];
    }
    $this->destinyType = $this->currentConfig["destiny_type"];
    $this->originType = $this->currentConfig["origin_type"];
    switch ($this->currentConfig["origin_type"]) {
      case 'node':
        $this->getBaseNodeValues();
        break;
      case 'taxonomy_term':
        $this->getBaseVocavularyValues();
        break;
      case 'user':
        $this->getBaseUserValues();
        break;
      case 'field_collection':
        $this->getBaseFieldCollectionValues();
        break;
      default:
    }

    $this->configFactory = $config_factory;
    $this->fieldTypeMWManager = $field_type_mw_manager;

    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration, $state);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration = NULL) {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('state'),
      $container->get('config.factory'),
      $container->get('plugin.manager.field_type_mw')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function query() {

    if($this->currentConfig["origin_type"] == 'node'){
      $this->query->condition('n.language', $this->translation, 'in');
    }

    foreach ($this->currentConfig[array_key_first($this->currentConfig)] as $field_name => $data_field) {
      if ($field_name != 'languages' && $field_name != 'i18n_mode' && $data_field['type'] != 'image' && $data_field['type'] != 'file' && $data_field["destiny"] != 'none' && !array_key_exists($field_name, $this->query->getTables())) {
        $this->addQuery($field_name, $data_field, $this->translation);
      }
    }
    if (isset($this->currentConfig["import_path_alias"]) && $this->currentConfig["import_path_alias"] == 1 && !array_key_exists('ual', $this->query->getTables())) {
      $this->queryEntityUrlAlias();
    }
    if (isset($this->currentConfig["source_type_date_field"]) && $this->currentConfig["source_type_date_field"] != NULL) {
      $this->addFromCreate(strtotime($this->currentConfig["source_type_date_field"]));
    }

    if (isset($this->currentConfig["import_only_ids"]) && $this->currentConfig["import_only_ids"] != NULL) {
      $this->importOnlyIds();
    }

    if (isset($this->currentConfig["limit_import"]) && $this->currentConfig["limit_import"] != NULL) {
      $this->limitImport();
    }

    $a = $this->query->execute()->fetchAll();
//    print_r($this->query->__toString());
//    die();
    return $this->query;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {

    $queryAdded = FALSE;

    switch ($this->currentConfig["origin_type"]) {
      case 'node':
        $this->query = $this->select('node', 'n');
        $this->query->condition('n.nid', $row->getSourceProperty('nid'));
        break;
      case 'taxonomy_term':
        $this->query = $this->select('taxonomy_vocabulary', 'tv');
        $this->query->condition('tv.vid', $row->getSourceProperty('vid'));
        if($this->destinyType == 'node'){
          $row->setSourceProperty('status', 1);
          $row->setSourceProperty('created', time());
        }
        break;
      default:
        break;
    }

    $translation = ['und', 'es'];
    if(isset($this->currentConfig[array_key_first($this->currentConfig)]["languages"])){
      $translation = ['und', $this->currentConfig[array_key_first($this->currentConfig)]["languages"]["default"]];
    }
    if(isset($this->configuration["constants"]["translations"]) && $this->configuration["constants"]["translations"]){
      $translation = $this->configuration["constants"]["langcode"];
    }
    if($this->currentConfig["origin_type"] == 'node'){
      $this->query->condition('n.language', $translation, 'in');
    }
    if(!in_array('und', $this->translation) && isset($this->currentConfig["i18n_mode"])){
      switch($this->currentConfig["i18n_mode"]){
        case 1:
          $ltt = $row->getSourceProperty('ltt');
          $ltt = explode('||', $ltt);
          $i18nc = $row->getSourceProperty('i18nc');
          $i18nc = explode('||', $i18nc);
          $i18nc2 = $row->getSourceProperty('i18nc_2');
          $i18nc2 = explode('||', $i18nc2);
          foreach($i18nc as $key => $value){
            $value_explode = explode(':', $value);
            $row->setSourceProperty($value_explode[2], $ltt[$key]);
            if($i18nc2[$key] != ''){
              $row->setSourceProperty('format', $i18nc2[$key]);
            }
          }
          break;
        case 4:
          break;
      }
    }
    foreach ($this->currentConfig[array_key_first($this->currentConfig)] as $field_name => $data_field) {
      if(isset($data_field['type']) && $data_field["destiny"] != 'none'){
        switch ($data_field['type']) {
          case 'image':
          case 'file':
            $queryAdded = TRUE;
            $this->addQuery($field_name, $data_field, $translation);
            break;
          case 'datetime':
            $date = $row->getSourceProperty($field_name . '_value');
            if (isset($data_field['format']) && $data_field['format'] != '' && $date != '') {
              $date = DrupalDateTime::createFromFormat('Y-m-d H:i:s', $date);
              $date->format($data_field['format']);
              $row->setSourceProperty($field_name . '_value', $date->format($data_field['format']));

            }
            break;
          case 'list_text':
            if ($data_field['destiny_type'] == 'entity_reference') {
              $this->listTextToTaxonomy($row, $field_name, $data_field);
            }
            break;
        }
      }
    }

    if($queryAdded){
      $valuesToRow = $this->query->execute()->fetchAll();

      foreach($valuesToRow[0] as $key => $value){
        if($value != ''){
          $row->setSourceProperty($key, $value);
        }
      }
    }


    // Add language field.
//    $bundle = $row->getSourceProperty('field_name');
//    $bundle = substr($bundle, FieldCollection::FIELD_COLLECTION_PREFIX_LENGTH);
//    $row->setSourceProperty('bundle', $bundle);
//
//    // Get Field API field values.
//    $field_names = array_keys($this->getFields('field_collection_item', $row->getSourceProperty('field_name')));
//    $fields = $this->getFields('field_collection_item', $row->getSourceProperty('field_name'));
//
//    $item_id = $row->getSourceProperty('item_id');
//    $revision_id = $row->getSourceProperty('revision_id');
//    $language = $row->getSourceProperty('language');
//
//    foreach ($field_names as $field_name) {
//      $field_language = $fields[$field_name]['translatable'] ? $language : NULL;
//      $value = $this->getFieldValues('field_collection_item', $field_name, $item_id, $revision_id, $field_language);
//      $row->setSourceProperty($field_name, $value);
//    }









    return parent::prepareRow($row);
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    switch ($this->destinyType) {
      case 'node':
        $fields = [
          'nid' => $this->t('ID of node'),
          'title' => $this->t('Title of node'),
        ];
        break;

      case 'taxonomy_term':
        $fields = [
          'tid' => $this->t('The term ID.'),
          'vid' => $this->t('Existing term VID'),
          'name' => $this->t('The name of the term.'),
        ];
        break;
    }

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {

    switch ($this->configuration["origin_type"]) {
      case 'node':
        if(in_array('und', $this->translation)){
          $ids['nid'] = ['type' => 'integer', 'alias' => 'n'];
        }else{
          $ids['tnid'] = ['type' => 'integer', 'alias' => 'n'];
        }
        break;
      case 'taxonomy_term':
        if(in_array('und', $this->translation)){
          $ids['tid']['type'] = 'integer';
        }else{
          switch($this->currentConfig["i18n_mode"]){
            case 1:
              $ids['tid']['type'] = 'integer';
              break;
            case 4:
              $ids['tid'] = ['type' => 'integer', 'alias' => 'ttd_2'];
          }
        }
        break;
      case 'user':
        $ids['uid'] = ['type' => 'integer', 'alias' => 'u'];
        break;
      case 'field_collection':
        $ids['item_id'] = ['type' => 'integer', 'alias' => 'cfi'];
        break;
    }
    return $ids;
  }

  /**
   * {@inheritdoc}
   */
  private function listTextToTaxonomy(&$row, $field_name, $data_field): void {
    // Load taxonomy term.
    $term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadByProperties(['name' => $row->getSourceProperty($field_name . '_value')]);
    $term = reset($term);
    if ($term == FALSE) {
      $term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->create([
        'name' => $row->getSourceProperty($field_name . '_value'),
        'vid' => $data_field['destiny'],
      ]);
      $term->save();
    }
    $row->setSourceProperty($field_name . '_value', $term->id());
  }

  /**
   * {@inheritdoc}
   */
  protected function setUpDatabase(array $database_info) {
    $current_config = $this->config->get('migrate_wizard_settings');
    $key = getenv($current_config["migrate_db_key"]);
    $target = getenv($current_config["migrate_db_target"]);
    return Database::getConnection($target, $key);
  }

  /**
   * {@inheritdoc}
   */
  protected function getLeftJoinTable() {
    return "$this->leftTable.$this->leftTableId";
  }

  /**
   * {@inheritdoc}
   */
  protected function getBaseNodeValues() {
    $this->leftTable = 'n';
    $this->leftTableId = 'nid';
    $this->entityType = 'node';
    $this->leftJoinTable = $this->getLeftJoinTable();
    $this->query = $this->select('node', 'n');
    if ($this->destinyType == 'node') {
      $fields = [
        'title',
        'nid',
        'uid',
        'status',
        'created',
        'language',
        'tnid'
      ];
    }
    else {
      $fields = [
        'title',
        'uid',
        'status',
        'created',
      ];
      $this->query->addExpression("n.nid", 'nid', []);
      $this->query->groupBy('nid');
    }
    $alias = 'n';
    if(in_array('und', $this->translation)){
      $this->query->where('[n].[tnid] = 0 OR [n].[tnid] = [n].[nid]');
    }else{
      $this->query->where('[n].[tnid] <> 0 AND [n].[tnid] <> [n].[nid]');
    }
    foreach ($fields as $field) {
      $this->query->fields($alias, [$field]);
      $this->query->groupBy($field);
    }

    $this->query->isNotNull('n.nid');
    $this->query->condition('n.title', '', '<>');
    $this->query->condition('n.type', $this->bundle);
  }

  /**
   * {@inheritdoc}
   */
  protected function getBaseVocavularyValues() {

    $this->leftTable = 'tv';
    $this->leftTableId = 'vid';
    $this->entityType = 'taxonomy_term';
    $this->leftJoinTable = $this->getLeftJoinTable();
    $this->query = $this->select('taxonomy_vocabulary', 'tv');
    $fields = [
      'tid',
      'vid',
      'name',
      'description',
      'weight',
      'format',
      'language',
      'i18n_tsid',
    ];
    $table = 'taxonomy_term_data';
    $alias = 'ttd';

    $translations = implode("','", $this->translation);
    if(in_array('und', $this->translation)){
      $this->query->leftJoin($table, $alias, "$this->leftJoinTable = $alias.vid AND $this->leftTable.machine_name = '$this->bundle' AND ttd.language in('$translations')");
    }
    foreach ($fields as $field) {
      $this->query->fields($alias, [$field]);
      $this->query->groupBy($field);
    }

    $this->query->isNotNull('ttd.vid');
    $this->query->condition('ttd.name', '', '<>');

    if(!in_array('und', $this->translation) && isset($this->currentConfig["i18n_mode"])){
      switch($this->currentConfig["i18n_mode"]){
        case 1:
          $this->query->leftJoin($table, $alias, "$this->leftJoinTable = $alias.vid AND $this->leftTable.machine_name = '$this->bundle'");
          $this->query->leftJoin('i18n_string', 'i18n', "ttd.tid = i18n.objectindex");
          $this->query->leftJoin('locales_target', 'lt', "lt.lid = i18n.lid AND lt.language in('$translations')");
          $this->query->addExpression("GROUP_CONCAT(DISTINCT COALESCE(lt.translation, '') ORDER BY i18n.lid ASC SEPARATOR '||')", 'ltt', []);
          $this->query->addExpression("GROUP_CONCAT(DISTINCT COALESCE(i18n.context, '') ORDER BY i18n.lid ASC SEPARATOR '||')", 'i18nc', []);
          $this->query->addExpression("GROUP_CONCAT(DISTINCT COALESCE(i18n.format, '') ORDER BY i18n.lid ASC SEPARATOR '||')", 'i18nc', []);
//          $this->query->fields('i18n', ['format']);
          $this->query->condition('lt.translation', '', '<>');
//          $this->query->condition('i18n.format', '', '<>');
          break;
        case 4:
          $this->query->leftJoin($table, $alias, "$this->leftJoinTable = $alias.vid AND $this->leftTable.machine_name = '$this->bundle' AND ttd.language in('$translations')");
          $this->query->leftJoin($table, $alias.'_2', "ttd.i18n_tsid = {$alias}_2.i18n_tsid AND ttd.language in('$translations')");
          $this->query->fields($alias.'_2', ['tid']);
          $this->query->groupBy('ttd_2_tid');
          break;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseUserValues(): void
  {

    $this->leftTable = 'u';
    $this->leftTableId = 'uid';
    $this->entityType = 'user';
    $this->leftJoinTable = $this->getLeftJoinTable();
    $this->query = $this->select('users', 'u');
    $this->query->fields('u', [
      'name',
      'pass',
      'mail',
      'uid',
      'status',
      'access',
      'created',
      'login',
    ]);
    $this->query->condition('u.name', '', '<>');
    $this->query->condition('u.uid', 0, '<>');
    $this->query->condition('u.uid', 1, '<>');
    $this->query->isNotNull('u.name');
    $this->query->groupBy('u.uid');
    $this->query->groupBy('u.name');
    $this->query->groupBy('u.pass');
    $this->query->groupBy('u.mail');
    $this->query->groupBy('u.status');
    $this->query->groupBy('u.access');
    $this->query->groupBy('u.created');
    $this->query->groupBy('u.login');

    $this->query->leftJoin('users_roles', 'ur', "ur.uid = u.uid");
    $this->query->addExpression("GROUP_CONCAT(DISTINCT COALESCE(ur.rid, '') SEPARATOR '||')", 'roles', []);

    if (isset($this->currentConfig["import_cas"]) && $this->currentConfig["import_cas"]) {
      $this->queryUserCas();
    }

  }

  protected function queryUserCas(): void
  {

    $this->query->leftJoin('cas_user', 'cu', "$this->leftJoinTable = cu.uid");
    $this->query->fields('cu', ['cas_name']);
    $this->query->groupBy('cu.cas_name');
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseFieldCollectionValues(): void
  {

    $this->leftTable = 'cfi';
    $this->leftTableId = 'item_id';
    $this->entityType = 'field_collection_item';
    $this->leftJoinTable = $this->getLeftJoinTable();
    $this->query = $this->select('field_collection_item', 'cfi');
    $this->query->fields('cfi', [
      'item_id',
      'field_name'
    ]);
    $this->query->groupBy('cfi.item_id');
    $this->query->groupBy('cfi.field_name');
    $this->query->condition('cfi.field_name', $this->bundle);


    $this->query->leftJoin('field_data_'.$this->bundle, 'fd_'.$this->bundle, "fd_{$this->bundle}.{$this->bundle}_value = cfi.item_id");
    $this->query->leftJoin('node', 'n', "fd_{$this->bundle}.entity_id = n.nid");
    $this->query->addField('n', 'language');


  }

  /**
   * {@inheritdoc}
   */
  public function addQuery($field_name, $data_field, $translation, $fields_group = NULL): void
  {
    $field_type_mw = $this->fieldTypeMWManager->createCustomInstance($data_field['type'], $this->migration, $this->state);
    $field_type_mw->addQueryByType($field_name, 'field_data_' . $field_name, $data_field, $this, $translation, $fields_group);
  }

  /**
   * {@inheritdoc}
   */
  public function getDestinyType(){
    return $this->currentConfig['destiny_type'];
  }

  /**
   * {@inheritdoc}
   */
  public function getValue($value){
    return $this->{$value};
  }
  protected function addFromCreate($timestamp): void
  {
    $this->query->condition('created', $timestamp, '>=');
  }

  protected function queryEntityUrlAlias(): void
  {

    $translations = implode("','", $this->translation);
    $id__left = $this->leftJoinTable;
    if($this->leftJoinTable == 'tv.vid'){
      $id__left = 'ttd.tid';
    }

    $source = $this->entityType;
    if($this->entityType == 'taxonomy_term'){
      $source = 'taxonomy/term';
    }

    $this->query->leftJoin('url_alias', 'ual', "CONCAT('{$source}/', $id__left) = ual.source AND ual.pid = (SELECT MAX(ual2.pid) FROM url_alias as ual2 WHERE ual2.source=CONCAT('{$source}/', $id__left)) AND ual.language in('$translations')");
    $this->query->fields('ual', ['alias']);
    $this->query->groupBy('ual.alias');
  }

  protected function importOnlyIds(): void
  {
    switch($this->currentConfig["origin_type"]){
      case 'node':
        $this->query->condition('n.nid', explode(',', $this->currentConfig["import_only_ids"]), 'IN');
        break;
      case 'taxonomy_term':
        $this->query->condition('ttd.tid', explode(',', $this->currentConfig["import_only_ids"]), 'IN');
        break;
      case 'user':
        $this->query->condition('u.uid', explode(',', $this->currentConfig["import_only_ids"]), 'IN');
        break;
    }
  }

  protected function limitImport(): void
  {
    $this->query->range(0, $this->currentConfig["limit_import"]);

    $this->query->orderBy($this->leftJoinTable, 'ASC');
  }
}
